@@ -13,11 +13,9 @@
 #include <KJobUiDelegate>
 #include <KLocalizedString>
 #include <KMessageBox>
+#include <KNS3/QtQuickDialogWrapper>
 #include <KTar>
 
-#include <knewstuff_version.h>
-#include <QtQuickDialogWrapper>
-
 #include <QDir>
 #include <QDirIterator>
 #include <QFile>
@@ -425,70 +423,72 @@ QSet<QString> AppearanceSettings::extractKnsSkinIds(const QStringList &fileList)
 void AppearanceSettings::getNewSkins()
 {
     QPointer<KNS3::QtQuickDialogWrapper> dialog = new KNS3::QtQuickDialogWrapper(m_knsConfigFileName, this);
-    const QList<KNSCore::EntryInternal> changedEntries = dialog->exec();
-
-    quint32 invalidEntryCount = 0;
-    QString invalidSkinText;
-    for (const auto &_entry : changedEntries) {
-        if (_entry.status() != KNS3::Entry::Installed) {
-            continue;
-        }
-        const KNSCore::EntryInternal &entry = _entry;
-        bool isValid = true;
-        const QSet<QString> &skinIdList = extractKnsSkinIds(entry.installedFiles());
-
-        // Validate all skin IDs as each archive can contain multiple skins.
-        for (const QString &skinId : skinIdList) {
-            // Validate the current skin.
-            if (!validateSkin(skinId, true)) {
-                isValid = false;
+    dialog->open();
+    connect(dialog, &KNS3::QtQuickDialogWrapper::closed, this, [this, dialog] {
+        const QList<KNSCore::EntryInternal> changedEntries = dialog->changedEntries();
+        quint32 invalidEntryCount = 0;
+        QString invalidSkinText;
+        for (const auto &_entry : changedEntries) {
+            if (_entry.status() != KNS3::Entry::Installed) {
+                continue;
             }
-        }
+            const KNSCore::EntryInternal &entry = _entry;
+            bool isValid = true;
+            const QSet<QString> &skinIdList = extractKnsSkinIds(entry.installedFiles());
 
-        // We'll add an error message for the whole KNS entry if
-        // the current skin is marked as invalid.
-        // We should not do this per skin as the user does not know that
-        // there are more skins inside one archive.
-        if (!isValid) {
-            invalidEntryCount++;
-
-            // The user needs to know the name of the skin which
-            // was removed.
-            invalidSkinText += QString(QStringLiteral("<li>%1</li>")).arg(entry.name());
-
-            // Then remove the skin.
-            const QStringList files = entry.installedFiles();
-            for (const QString &file : files) {
-                QFileInfo info(QString(file).remove(QStringLiteral("/*")));
-                if (!info.exists()) {
-                    continue;
+            // Validate all skin IDs as each archive can contain multiple skins.
+            for (const QString &skinId : skinIdList) {
+                // Validate the current skin.
+                if (!validateSkin(skinId, true)) {
+                    isValid = false;
                 }
-                if (info.isDir()) {
-                    QDir(info.absoluteFilePath()).removeRecursively();
-                } else {
-                    QFile::remove(info.absoluteFilePath());
+            }
+
+            // We'll add an error message for the whole KNS entry if
+            // the current skin is marked as invalid.
+            // We should not do this per skin as the user does not know that
+            // there are more skins inside one archive.
+            if (!isValid) {
+                invalidEntryCount++;
+
+                // The user needs to know the name of the skin which
+                // was removed.
+                invalidSkinText += QString(QStringLiteral("<li>%1</li>")).arg(entry.name());
+
+                // Then remove the skin.
+                const QStringList files = entry.installedFiles();
+                for (const QString &file : files) {
+                    QFileInfo info(QString(file).remove(QStringLiteral("/*")));
+                    if (!info.exists()) {
+                        continue;
+                    }
+                    if (info.isDir()) {
+                        QDir(info.absoluteFilePath()).removeRecursively();
+                    } else {
+                        QFile::remove(info.absoluteFilePath());
+                    }
                 }
             }
         }
-    }
 
-    // Are there any invalid entries?
-    if (invalidEntryCount > 0) {
-        failInstall(xi18ncp("@info",
-                            "The following skin is missing required files. Thus it was removed:<ul>%2</ul>",
-                            "The following skins are missing required files. Thus they were removed:<ul>%2</ul>",
-                            invalidEntryCount,
-                            invalidSkinText));
-    }
+        // Are there any invalid entries?
+        if (invalidEntryCount > 0) {
+            failInstall(xi18ncp("@info",
+                                "The following skin is missing required files. Thus it was removed:<ul>%2</ul>",
+                                "The following skins are missing required files. Thus they were removed:<ul>%2</ul>",
+                                invalidEntryCount,
+                                invalidSkinText));
+        }
 
-    if (!changedEntries.isEmpty()) {
-        // Reset the selection in case the currently selected
-        // skin was removed.
-        resetSelection();
+        if (!changedEntries.isEmpty()) {
+            // Reset the selection in case the currently selected
+            // skin was removed.
+            resetSelection();
 
-        // Re-populate the list of skins if the user changed something.
-        populateSkinList();
-    }
+            // Re-populate the list of skins if the user changed something.
+            populateSkinList();
+        }
 
-    delete dialog;
+        dialog->deleteLater();
+    });
 }
